Package: dblog
Title: Logging for DBI
Version: 0.0.0.9013
Date: 2023-10-09
Authors@R: 
    person(given = "Kirill",
           family = "Müller",
           role = c("aut", "cre"),
           email = "krlmlr+r@mailbox.org",
           comment = c(ORCID = "0000-0002-1416-3412"))
Description: Provides logging of DBI methods for arbitrary backends.
License: GPL-3
URL: https://dblog.r-dbi.org, https://github.com/r-dbi/dblog
BugReports: https://github.com/r-dbi/dblog/issues
Imports: 
    collections (>= 0.1.4),
    DBI,
    glue,
    methods,
    purrr,
    rlang
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.2.3
Collate: 
    'Driver.R'
    'Connection.R'
    'Result.R'
    'deparse.R'
    'import.R'
    'make-log-call.R'
Suggests: 
    testthat (>= 3.0.0)
Config/testthat/edition: 3
