# Generated by roxygen2: do not edit by hand

S3method(format,dblogConnection)
export(dblog)
export(dblog_cnr)
export(get_default_logger)
export(make_collect_logger)
export(make_text_logger)
import(DBI)
import(methods)
import(rlang)
